@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -250,11 +250,11 @@ void xine_profiler_print_results (void) XINE_PROTECTED;
 #  define xine_container_of(ptr, type, member)           \
   ({                                                     \
      const typeof(((type *)0)->member) *__mptr = (ptr);  \
-     (type *)((char *)__mptr - offsetof(type, member));  \
+     (type *)(void *)((char *)__mptr - offsetof(type, member));  \
   })
 #else
 #  define xine_container_of(ptr, type, member) \
-  ((type *)((char *)(1 ? (ptr) : &((type *)0)->member) - offsetof(type, member)))
+  ((type *)(void *)((char *)(1 ? (ptr) : &((type *)0)->member) - offsetof(type, member)))
 #endif
 
 /*
@@ -532,6 +532,17 @@ void _x_nv12_to_yv12(const uint8_t *y_src,  int y_src_pitch,
                      uint8_t *u_dst, int u_dst_pitch,
                      uint8_t *v_dst, int v_dst_pitch,
                      int width, int height) XINE_PROTECTED;
+void _x_yv12_to_nv12(const uint8_t *y_src, int y_src_pitch,
+                     const uint8_t *u_src, int u_src_pitch,
+                     const uint8_t *v_src, int v_src_pitch,
+                     uint8_t *y_dst,  int y_dst_pitch,
+                     uint8_t *uv_dst, int uv_dst_pitch,
+                     int width, int height) XINE_PROTECTED;
+void _x_yuy2_to_nv12(const uint8_t *src_yuy2_map, int yuy2_pitch,
+                     uint8_t *y_dst,  int y_dst_pitch,
+                     uint8_t *uv_dst, int uv_dst_pitch,
+                     int width, int height) XINE_PROTECTED;
+
 
 /* print a hexdump of the given data */
 void xine_hexdump (const void *buf, int length) XINE_PROTECTED;
@@ -734,6 +745,28 @@ int xine_monotonic_clock(struct timeval *tv, struct timezone *tz) XINE_PROTECTED
 void _x_report_video_fourcc (xine_t *, const char *module, uint32_t) XINE_PROTECTED;
 void _x_report_audio_format_tag (xine_t *, const char *module, uint32_t) XINE_PROTECTED;
 
+/** fast strings are either inside an application supplied buffer,
+ *  or they are (re)allocated by xine_fast_string_set ().
+ *  the returned pointer will be at least 8 byte aligned,
+ *  and can be read like an ordinary C string. */
+#define XINE_FAST_STRING 1
+/** return the byte size needed for an application supplied buffer. */
+size_t xine_fast_string_need (size_t max_strlen) XINE_PROTECTED;
+/** return the actual max strlen. */
+size_t xine_fast_string_max (char *fast_string) XINE_PROTECTED;
+/** set up a fast string inside an application supplied buffer. */
+char *xine_fast_string_init (char *buf, size_t bsize) XINE_PROTECTED;
+/** set or change the contents of a fast string.
+ *  fast_text may be NULL, to allocate a new one.
+ *  you can even edit the string manually, then apply the new size with text == NULL. */
+char *xine_fast_string_set (char *fast_string, const char *text, size_t tsize) XINE_PROTECTED;
+/** fast strcmp ().
+ *  BTW: XINE_PROTECTED does not link on some systems when doing
+ *  xine_sarray_new (size, (xine_sarray_comparator_t)xine_fast_string_cmp);. */
+int xine_fast_string_cmp (char *fast_string1, char *fast_string2);
+/** free a fast string if it is not application supplied. */
+void xine_fast_string_free (char **fast_string) XINE_PROTECTED;
+
 /* don't harm following code */
 #ifdef extern
 #  undef extern
